<?php
  #
  # Program id     : EMGRC.inc.php
  #
  # subject        : uemailb?z client side program
  #
  #
  //
  // $Id: EMGRC.inc.php,v 20030725-1350 2003/07/25 13:50:55 daniel Exp $

  require_once('Net/Socket.php');
  /*
   *
   * EMGRC Access Class
   *
   * For usage see the example script
   */

  define('NET_EMGRC_STATE_DISCONNECTED',  1, true);
  define('NET_EMGRC_STATE_AUTHORISATION', 2, true);
  define('NET_EMGRC_STATE_TRANSACTION',   4, true);

  class Net_EMGRC 
  {

    /*
    * Timeout that is passed to the socket object
    *
    * @var integer
    */
    var $_timeout;

    /*
     * Socket object
     *
     * @var object
     */
    var $_socket;

    /*
     * Current state of the connection. Used with the
     * constants defined above.
     *
     * @var integer
     */
    var $_state;

    /*
     * Hostname to connect to
     *
     * @var string
     */
    var $_host;

    /*
     * Port to connect to
     *
     * @var integer
     */
    var $_port;

    /*
     * error massage     
     *
     * @var string  
     */
    var $_error;


    /*
     * Constructor. Sets up the object variables, and instantiates
     * the socket object.
     *
     */
    function Net_EMGRC()
    {
      $this->_timeout   =  30;
      $this->_state     =  NET_EMGRC_STATE_DISCONNECTED;
      $this->_socket    =& new Net_Socket();
    }

    /*
     * Connects to the given host on the given port.
     * Also looks for the timestamp in the greeting
     * needed for APOP authentication
     *
     * @param  $host Hostname/IP address to connect to
     * @param  $port Port to use to connect to on host
     * @return bool  Success/Failure
     */
    function connect($host = 'localhost', $port = 10038)
    {
      $this->_host = $host;
      $this->_port = $port;

      $result = $this->_socket->connect($host, $port, false, $this->_timeout);
      if ($result === true) 
      {
        $data = $this->_socket->readLine();
        $this->_error = $data;
        if  (@substr($data, 0, 3) == '+OK') 
        {
          $this->_state    = NET_EMGRC_STATE_AUTHORISATION;
          return true;
        }
      }
      $this->_socket->disconnect();
      return false;
    }

    /*
     * Disconnect function. Sends the QUIT command
     * and closes the socket.
     *
     * @return bool Success/Failure
     */
    function disconnect()
    {
      return $this->_cmdQuit();
    }

    /*
     * Performs the login procedure. If there is a timestamp
     * stored, APOP will be tried first, then basic USER/PASS.
     *
     * @param  $user Username to use
     * @param  $pass Password to use
     * @param  $apop Whether to try APOP first
     * @return bool  Success/Failure
     */
    function login($user, $pass)
    {
      if  ($this->_state == NET_EMGRC_STATE_AUTHORISATION) 
      {
        if  ($this->_cmdUser($user) AND $this->_cmdPass($pass)) 
        {
          $this->_state = NET_EMGRC_STATE_TRANSACTION;
          return true;
        }
      }
      return false;
    }


    /*
     * Sends the USER command
     *
     * @param  $user Username to send
     * @return bool  Success/Failure
     */
    function _cmdUser($user)
    {
      if  ($this->_state == NET_EMGRC_STATE_AUTHORISATION) 
      {
        return (bool)$this->_sendCmd('USER ' . $user);
      }
      return false;
    }


    /*
     * Sends the PASS command
     *
     * @param  $pass Password to send
     * @return bool  Success/Failure
     */
    function _cmdPass($pass)
    {
      if  ($this->_state == NET_EMGRC_STATE_AUTHORISATION) 
      {
        return (bool)$this->_sendCmd('PASS ' . $pass);
      }
      return false;
    }


    /*
     * Sends the QUIT command
     *
     * @return bool Success/Failure
     */
    function _cmdQuit()
    {
      $data = $this->_sendCmd('QUIT');
      $this->_state = NET_POP3_STATE_DISCONNECTED;
      $this->_socket->disconnect();
      return (bool)$data;
    }
    
    /*
     * Sends the CHPW
     *
     * @param  $password Changed new password
     * @return bool Success/Failure
     */
    function ChangePassWord($password)
    {
      if  ($this->_state == NET_EMGRC_STATE_TRANSACTION)
      {
        return (bool)$this->_sendCmd('CHPW ' . $password);
      }
      return false;
    }

    /*
     * Sends the DELM
     *
     *   Delete Server Mail Box
     * @param none
     * @return bool Success/Failure
     */
    function DeleteMBox()
    {
      if  ($this->_state == NET_EMGRC_STATE_TRANSACTION)
      {
        return (bool)$this->_sendCmd('DELM ');
      }
      return false;
    }
    /*
     * Sends a command, checks the reponse, and
     * if good returns the reponse, other wise
     * returns false.
     *
     * @param  $cmd  Command to send (\r\n will be appended)
     * @return mixed First line of response if successful, otherwise false
     */
    function _sendCmd($cmd)
    {
      $result = $this->_socket->writeLine($cmd);
      if  (!PEAR::isError($result) AND $result)
      {
        $data = $this->_socket->readLine();
        $this->_error = $data;
        if (!PEAR::isError($data) AND substr($data, 0, 3) == '+OK') 
        {
          return $data;
        }
      }
      return false;
    }
  }
  //
  // authentication
  //
  function CheckPassword(
                         $hostname,
                         $port,
                         $username,
                         $password
                        )
  {
    $emgrc =& new Net_EMGRC();
    if  (!is_object($emgrc))
    {
      return('4Cannot create Socket object!');
    }
    if  ($emgrc->connect($hostname, $port)==FALSE)
    {
      return('3' . $emgrc->_error);
    }
    if  ($emgrc->login($username, base64_encode($password))==FALSE)
    {
      return('2' . $emgrc->_error);
    }
    $emgrc->disconnect();
    return("");
  }
?>
